/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 */

#pragma once

#include <stdbool.h>

#include "stringlist.h"

struct daemon;
typedef struct daemon daemon_t;
struct daemon_node;
typedef struct daemon_node daemon_node_t;
struct graph;
typedef struct graph graph_t;

// node in a linked list of daemons.
struct daemon_node
{
	daemon_node_t* next;
	daemon_t* daemon;
};

// daemon - process or service managed by the RC init system.
struct daemon
{
	// link to next daemon (root level)
	daemon_t* next;
	// path to RC script
	const char* script;
	// before list
	string_node_t* before;
	// keywords used for filtering.
	string_node_t* keywords;
	// provides list
	string_node_t *provides;
	// requirements list
	string_node_t* requires;
	// list of daemons expected to run before this daemon.
	daemon_node_t *parents;
	// list of daemons that may run after this daemon.
	daemon_node_t* children;
};

// create a daemon - script is full path to the RC script (usually /etc/rc.d/* or /usr/local/etc/rc.d/*).
daemon_t *daemon_create(graph_t *graph, const char *script);
// add 'parent' daemon to 'daemon's list of parents.
void daemon_parent(daemon_t *daemon, daemon_t *parent);
// add 'child' daemon to 'daemon's list of children.
void daemon_child(daemon_t* daemon, daemon_t* child);
// remove a 'parent' daemon because it has run (child no longer waiting on it).
void daemon_satisfy(daemon_t* daemon, daemon_t* complete);
// parse one line of text from an RC script for the specified daemon.
// this fills out the before, keywords, provides, and requires string lists.
void daemon_parse(daemon_t* daemon, const char* line);
// clean up
void daemon_destroy(daemon_t* daemon);
// return true if daemon has no parent dependencies.
bool daemon_emancipated(daemon_t* daemon);
// return true if one or more keys matches the daemon's keyword list.
bool daemon_has_one_or_more_keywords(daemon_t* daemon, string_node_t* keywords);

bool daemon_has_parent(daemon_t* daemon, daemon_t* parent);
