/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 */

#pragma once

#include "hash.h"

/* Directed graph of daemons in a parent / child hierarchy. */
struct graph
{
	/* hash map of named provider daemons. */
	hash_t providers;
	/* keep list - keep daemons that match these keywords. */
	string_node_t* keep;
	/* skip list - skip daemons that match these keywords. */
	string_node_t* skip;
	/* concurrency limit. */
	unsigned limit;
	/* root node in the linked list of all daemons. */
	daemon_t* root;
	/* number of daemons in the root list. */
	unsigned count;
	/* name of this program (for logging). */
	const char* name;
};

void graph_init(graph_t* graph);
int graph_parse(graph_t* graph, int argc, const char *argv[]);
bool graph_resolve(graph_t* graph);
unsigned graph_group_print(graph_t* graph);
unsigned graph_group_list(graph_t* graph, daemon_t *list[]);
