/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 */

#pragma once

#include <iostream>
#include <math.h>

#ifndef _countof
#define _countof(arg) ( (sizeof arg) / (sizeof arg[0]) )
#endif

// poor-man's unit testing framework.
struct ScopeTest
{
	bool pass = true;
	const char *name;
	ScopeTest(const char *func) : name(func) { };
	~ScopeTest() { std::cout << name << " " << (pass ? "passes" : "fails") << "." << std::endl; }
	
	void isTrue(bool truth, const char *expression)
	{
		if (!truth) { pass = false; std::cout << name << "(" << __LINE__ << ") " << expression << std::endl; } else ; // nothing
	}

	void isFalse(bool truth, const char *expression)
	{
		if (truth) { pass = false; std::cout << name << "(" << __LINE__ << ") " << expression << std::endl; } else ; // nothing
	}

	void AreEqual(int expected, int actual)
	{
		if (expected != actual) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") expected " << expected << " actual " << actual << std::endl; } else ; // nothing
	}
	void AreEqual(unsigned expected, unsigned actual)
	{
		if (expected != actual) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") expected " << expected << " actual " << actual << std::endl; } else ; // nothing
	}

	void AreEqual(long expected, long actual)
	{
		if (expected != actual) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") expected " << expected << " actual " << actual << std::endl; } else ; // nothing
	}

	void AreEqual(float expected, float actual)
	{
		if ( fabs(expected - actual) > 0.000001f ) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") expected " << expected << " actual " << actual << std::endl; } else ; // nothing
	}
	void AreEqual(const char *expected, const char *actual)
	{
		if ( strcmp(expected, actual) != 0 ) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") expected " << expected << " actual " << actual << std::endl; } else ; // nothing
	}
	
	void isNull(void *pointer, const char *expression)
	{
		if (nullptr != pointer) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") " << expression << std::endl; } else ; // nothing
	}

	void isNotNull(void *pointer, const char *expression)
	{
		if (nullptr == pointer) { pass = false; std::cout << __func__ << "(" << __LINE__ << ") " << expression << std::endl; } else ; // nothing
	}
};

#define IsTrue(expression) \
	isTrue(expression, #expression)

#define IsFalse(expression) \
	isFalse(expression, #expression)

#define IsNull(expression) \
	isNull(expression, #expression)

#define IsNotNull(expression) \
	isNotNull(expression, #expression)
