/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 */

#include "stringlist.h"
#include <stdlib.h>
#include <memory.h>
#include <string.h>

string_node_t *string_create_length(const char* text, unsigned len)
{
	string_node_t* node = (string_node_t*)malloc(sizeof(*node) + len);
	memcpy(node->text, text, len);
	node->text[len] = 0;
	node->next = NULL;
	return node;
}

string_node_t *string_create(const char* text)
{
	return string_create_length(text, (unsigned)strlen(text));
}

string_node_t* string_destroy(string_node_t* node)
{
	string_node_t* swap = node->next;
	free(node);
	return swap;
}

void string_destroy_list(string_node_t* node)
{
	while (node)
	{
		node = string_destroy(node);
	}
}

string_node_t* string_list_append_length(string_node_t* root, const char* text, unsigned len)
{
	string_node_t* node = root;
	while (node)
	{
		if (strncmp(node->text, text, len) == 0)
			return root;
		node = node->next;
	}
	node = string_create_length(text, len);
	node->next = root;
	return node;
}

string_node_t* string_list_append(string_node_t* root, const char* text)
{
	return string_list_append_length(root, text, (unsigned)strlen(text));
}

bool string_list_contains(const string_node_t* root, const char* text)
{
	while (root)
	{
		if (strcmp(root->text, text) == 0)
			return true;
		root = root->next;
	}
	return false;
}

bool string_lists_overlap(const string_node_t* left, const string_node_t *right)
{
	while (right)
	{
		if (string_list_contains(left, right->text))
			return true;
		right = right->next;
	}
	return false;
}
