/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 */

#pragma once

#include <stdbool.h>

struct string_node;
typedef struct string_node string_node_t;

/* string node in a linked list of strings. */
struct string_node
{
	string_node_t* next;
	char text[1];
};

/* create a string node from a unterminated string of predetermined length. */
string_node_t* string_create_length(const char* text, unsigned len);
/* create a string node from a nul terminated string. */
string_node_t* string_create(const char* text);
string_node_t* string_destroy(string_node_t* node);
/* append a string node from a unterminated string of predetermined length to the list. */
string_node_t* string_list_append_length(string_node_t* root, const char* text, unsigned len);
/* append a string node from a nul terminated string to the list. */
string_node_t* string_list_append(string_node_t* root, const char* text);
/* return true of any of the string nodes matches the given text. */
bool string_list_contains(const string_node_t* root, const char* text);
/* return true if any string node from the left list matches a string node from the right list. */
bool string_lists_overlap(const string_node_t* left, const string_node_t *right);
void string_destroy_list(string_node_t* node);

